package cz.muni.fi.pa165.web;

import cz.muni.fi.pa165.model.Book;
import cz.muni.fi.pa165.model.BookService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.util.UriComponentsBuilder;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.*;

/**
 * MVC Controller for Books.
 *
 *
 * @author Martin Kuba makub@ics.muni.cz
 */
@Controller
@RequestMapping("/book")
public class BookController {

    final static Logger log = LoggerFactory.getLogger(BookController.class);

    @Autowired
    private BookService bookService;


    @RequestMapping(value = "/list", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public List<Book> show(HttpServletResponse response) {
        log.debug("list()");
        return bookService.getAllBooks();
    }

    @RequestMapping(value = "/get/{id}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public Book get(@PathVariable int id, HttpServletResponse response) {
        log.debug("get({})", id);
        Book book = bookService.getBookById(id);
        if (book == null) {
            log.warn("book {} not found", id);
            response.setStatus(HttpServletResponse.SC_NOT_FOUND);
        }
        return book;
    }

    @RequestMapping(value = "/delete/{id}", method = {RequestMethod.POST, RequestMethod.DELETE}, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public Object delete(@PathVariable int id, HttpServletResponse response) {
        log.debug("delete({})", id);
        Book book = bookService.getBookById(id);
        if (book == null) {
            log.warn("book {} not found", id);
            response.setStatus(HttpServletResponse.SC_NOT_FOUND);
        } else {
            bookService.deleteBook(book);
        }
        Map<String, Object> map = new HashMap<>();
        map.put("id", id);
        return map;
    }

    @RequestMapping(value = "/update", method = {RequestMethod.POST, RequestMethod.PUT},
            produces = MediaType.APPLICATION_JSON_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public Object update(@RequestBody @Valid Book book, BindingResult bindingResult,
                         UriComponentsBuilder uriBuilder, HttpServletResponse response) throws IOException {
        log.debug("update({})", book);
        if (bindingResult.hasErrors()) {
            log.debug("binding errors");
            Map<String,Object> map = new HashMap<>();
            if(bindingResult.hasGlobalErrors()) {
                Map<String,String> fes = new HashMap<>();
                for (ObjectError ge : bindingResult.getGlobalErrors()) {
                    log.debug("FieldError: objectName={} codes={} args={} defmsg={}",
                            ge.getObjectName(), Arrays.asList(ge.getCodes()),Arrays.asList(ge.getArguments()), ge.getDefaultMessage());
                    fes.put(ge.getObjectName(), ge.getDefaultMessage());
                }
                map.put("globalErrors",fes);
            }
            if(bindingResult.hasFieldErrors()) {
                Map<String,String> fes = new HashMap<>();
                for (FieldError fe : bindingResult.getFieldErrors()) {
                    log.debug("FieldError: objectName={} field={} rejectedvalue={} codes={} args={} defmsg={}",
                            fe.getObjectName(), fe.getField(), fe.getRejectedValue(),
                            Arrays.asList(fe.getCodes()),Arrays.asList(fe.getArguments()),fe.getDefaultMessage());
                    fes.put(fe.getField(),fe.getDefaultMessage());
                }
                map.put("fieldErrors",fes);
            }
            log.debug("returns map={}",map);
            response.setStatus(422);
            return map;
        }
        if(book==null) {
            log.debug("null input");
            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            return null;
        }
        if(book.getId()==0) {
            bookService.createBook(book);
        } else {
            Book bookById = bookService.getBookById(book.getId());
            if (bookById == null) {
                log.warn("book {} not found", book.getId());
                response.setStatus(HttpServletResponse.SC_NOT_FOUND);
                return null;
            }
            bookService.updateBook(book);
        }
        //redirect not allowed for Cross-Origin Request with Preflight, return updated data directly
        return book;
    }

    @RequestMapping(value = "/add", method = {RequestMethod.POST, RequestMethod.PUT},
            produces = MediaType.APPLICATION_JSON_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public Object add(@RequestBody @Valid Book book, BindingResult bindingResult,
                      UriComponentsBuilder uriBuilder, HttpServletResponse response) throws IOException {
        log.debug("add({})",book);
        book.setId(0);
        return update(book,bindingResult,uriBuilder,response);
    }


}
